@@ -1,4 +1,6 @@
 use ratpack::prelude::*;
+#[cfg(feature = "unix")]
+use std::path::PathBuf;
 
 async fn hello(
     req: Request<Body>,
@@ -22,7 +24,17 @@ async fn main() -> Result<(), ServerError> {
     let mut app = App::new();
     app.get("/:name", compose_handler!(hello));
 
-    app.serve("127.0.0.1:3000").await?;
+    #[cfg(feature = "unix")]
+    {
+        std::fs::remove_file("/tmp/server.sock").unwrap_or_default();
+        eprintln!("Serving over /tmp/server.sock");
+        app.serve_unix(PathBuf::from("/tmp/server.sock")).await?;
+    }
+    #[cfg(not(feature = "unix"))]
+    {
+        eprintln!("Serving over 127.0.0.1:3000");
+        app.serve("127.0.0.1:3000").await?;
+    }
 
     Ok(())
 }
