@@ -4,6 +4,11 @@ use http::{HeaderMap, Method, Request, Response, StatusCode};
 use hyper::{server::conn::Http, service::service_fn, Body};
 use tokio::{net::TcpListener, sync::Mutex};
 
+#[cfg(feature = "unix")]
+use std::path::PathBuf;
+#[cfg(feature = "unix")]
+use tokio::net::UnixListener;
+
 use crate::{handler::Handler, router::Router, Error, ServerError, TransientState};
 
 /// App is used to define application-level functionality and initialize the server. Routes are
@@ -143,14 +148,17 @@ impl<S: 'static + Clone + Send, T: TransientState + 'static + Clone + Send> App<
         let _uri = req.uri().clone();
         let _method = req.method().clone();
 
-        #[cfg(feature = "logging")]
+        #[cfg(all(feature = "logging", not(feature = "trace")))]
         log::info!("{} request to {}", _method, _uri);
 
+        #[cfg(feature = "trace")]
+        tracing::info!("{} request to {}", _method, _uri);
+
         match self.router.dispatch(req, self.clone()).await {
             Ok(resp) => {
                 let _status = resp.status().clone();
 
-                #[cfg(feature = "logging")]
+                #[cfg(all(feature = "logging", not(feature = "trace")))]
                 log::info!(
                     "{} request to {}: responding with status {}",
                     _method,
@@ -158,16 +166,32 @@ impl<S: 'static + Clone + Send, T: TransientState + 'static + Clone + Send> App<
                     _status,
                 );
 
+                #[cfg(feature = "trace")]
+                tracing::info!(
+                    "{} request to {}: responding with status {}",
+                    _method,
+                    _uri,
+                    _status,
+                );
+
                 Ok(resp)
             }
             Err(e) => {
-                #[cfg(feature = "logging")]
+                #[cfg(all(feature = "logging", not(feature = "trace")))]
                 log::error!(
                     "{} request to {}: responding with error {:?}",
                     _method,
                     _uri,
                     e,
                 );
+
+                #[cfg(feature = "trace")]
+                tracing::error!(
+                    "{} request to {}: responding with error {:?}",
+                    _method,
+                    _uri,
+                    e,
+                );
                 match e.clone() {
                     Error::StatusCode(sc, msg) => Ok(Response::builder()
                         .status(sc)
@@ -182,6 +206,35 @@ impl<S: 'static + Clone + Send, T: TransientState + 'static + Clone + Send> App<
         }
     }
 
+    #[cfg(feature = "unix")]
+    pub async fn serve_unix(self, filename: PathBuf) -> Result<(), ServerError> {
+        let unix_listener = UnixListener::bind(filename)?;
+        loop {
+            let (stream, _) = unix_listener.accept().await?;
+
+            let s = self.clone();
+            let sfn = service_fn(move |req: Request<Body>| {
+                let s = s.clone();
+                async move { s.clone().dispatch(req).await }
+            });
+
+            tokio::task::spawn(async move {
+                if let Err(http_err) = Http::new()
+                    .http1_keep_alive(true)
+                    .serve_connection(stream, sfn)
+                    .await
+                {
+                    #[cfg(feature = "logging")]
+                    log::error!("Error while serving HTTP connection: {}", http_err);
+                    #[cfg(feature = "trace")]
+                    tracing::error!("Error while serving HTTP connection: {}", http_err);
+                    #[cfg(all(not(feature = "trace"), not(feature = "logging")))]
+                    eprintln!("Error while serving HTTP connection: {}", http_err);
+                }
+            });
+        }
+    }
+
     /// Start a TCP/HTTP server with tokio. Performs dispatch on an as-needed basis. This is a more
     /// common path for users to start a server.
     pub async fn serve(self, addr: &str) -> Result<(), ServerError> {
@@ -189,15 +242,21 @@ impl<S: 'static + Clone + Send, T: TransientState + 'static + Clone + Send> App<
 
         let tcp_listener = TcpListener::bind(socketaddr).await?;
         loop {
+            let (tcp_stream, sa) = tcp_listener.accept().await?;
+
             let s = self.clone();
-            let sfn = service_fn(move |req: Request<Body>| {
+            let sfn = service_fn(move |mut req: Request<Body>| {
+                let ip = sa.ip();
+                req.extensions_mut().insert(ip);
                 let s = s.clone();
                 async move { s.clone().dispatch(req).await }
             });
-            let (tcp_stream, _sa) = tcp_listener.accept().await?;
 
-            #[cfg(feature = "logging")]
-            log::trace!("Request from {}", _sa,);
+            #[cfg(all(feature = "logging", not(feature = "trace")))]
+            log::trace!("Request from {}", sa);
+
+            #[cfg(feature = "trace")]
+            tracing::trace!("Request from {}", sa);
 
             tokio::task::spawn(async move {
                 if let Err(http_err) = Http::new()
@@ -207,7 +266,9 @@ impl<S: 'static + Clone + Send, T: TransientState + 'static + Clone + Send> App<
                 {
                     #[cfg(feature = "logging")]
                     log::error!("Error while serving HTTP connection: {}", http_err);
-                    #[cfg(not(feature = "logging"))]
+                    #[cfg(feature = "trace")]
+                    tracing::error!("Error while serving HTTP connection: {}", http_err);
+                    #[cfg(all(not(feature = "trace"), not(feature = "logging")))]
                     eprintln!("Error while serving HTTP connection: {}", http_err);
                 }
             });
@@ -227,14 +288,21 @@ impl<S: 'static + Clone + Send, T: TransientState + 'static + Clone + Send> App<
         let config = tokio_rustls::TlsAcceptor::from(Arc::new(config));
         let tcp_listener = TcpListener::bind(socketaddr).await?;
         loop {
+            let (tcp_stream, sa) = tcp_listener.accept().await?;
+
             let s = self.clone();
-            let sfn = service_fn(move |req: Request<Body>| {
+            let sfn = service_fn(move |mut req: Request<Body>| {
+                let ip = sa.ip();
+                req.extensions_mut().insert(ip);
                 let s = s.clone();
                 async move { s.clone().dispatch(req).await }
             });
-            let (tcp_stream, _sa) = tcp_listener.accept().await?;
-            #[cfg(feature = "logging")]
-            log::trace!("Request from {}", _sa,);
+
+            #[cfg(all(feature = "logging", not(feature = "trace")))]
+            log::trace!("Request from {}", sa);
+
+            #[cfg(feature = "trace")]
+            tracing::trace!("Request from {}", sa);
 
             let config = config.clone();
             tokio::task::spawn(async move {
@@ -247,14 +315,18 @@ impl<S: 'static + Clone + Send, T: TransientState + 'static + Clone + Send> App<
                         {
                             #[cfg(feature = "logging")]
                             log::error!("Error while serving HTTP connection: {}", http_err);
-                            #[cfg(not(feature = "logging"))]
+                            #[cfg(feature = "trace")]
+                            tracing::error!("Error while serving HTTP connection: {}", http_err);
+                            #[cfg(all(not(feature = "trace"), not(feature = "logging")))]
                             eprintln!("Error while serving HTTP connection: {}", http_err);
                         }
                     }
                     Err(e) => {
                         #[cfg(feature = "logging")]
                         log::error!("Error while serving TLS: {:?}", e);
-                        #[cfg(not(feature = "logging"))]
+                        #[cfg(feature = "trace")]
+                        tracing::error!("Error while serving TLS: {:?}", e);
+                        #[cfg(all(not(feature = "trace"), not(feature = "logging")))]
                         eprintln!("Error while serving TLS: {:?}", e);
                     }
                 }
